@@ -16,11 +16,9 @@
 
 from __future__ import unicode_literals, absolute_import
 
-import pymongo
-
 from motor.motor_py2_compat import text_type
 
-version_tuple = (2, 0, 0)
+version_tuple = (2, 1, 0)
 
 
 def get_version_string():
@@ -30,19 +28,6 @@ def get_version_string():
 version = get_version_string()
 """Current version of Motor."""
 
-pymongo_required = 3, 4
-if pymongo.version_tuple[:2] < pymongo_required:
-    major, minor = pymongo_required
-    msg = (
-        "Motor %s requires PyMongo %s.%s or later. "
-        "You have PyMongo %s. "
-        "Do python -m pip install \"pymongo>=%s.%s,<4\""
-    ) % (version,
-         major, minor,
-         pymongo.version,
-         major, minor)
-
-    raise ImportError(msg)
 
 try:
     import tornado
@@ -50,7 +35,7 @@ except ImportError:
     tornado = None
 else:
     # For backwards compatibility with Motor 0.4, export Motor's Tornado classes
-    # at module root. This may change in Motor 1.0. First get __all__.
+    # at module root. This may change in the future. First get __all__.
     from .motor_tornado import *
 
     # Now some classes that aren't in __all__ but might be expected.
