@@ -36,18 +36,154 @@ read preference, and/or write concern from this :class:`MotorClient`.
   - `name`: The name of the database - a string.
   - `codec_options` (optional): An instance of
     :class:`~bson.codec_options.CodecOptions`. If ``None`` (the
-    default) the :attr:`codec_options` of this :class:`MongoClient` is
+    default) the :attr:`codec_options` of this :class:`MotorClient` is
     used.
   - `read_preference` (optional): The read preference to use. If
     ``None`` (the default) the :attr:`read_preference` of this
-    :class:`MongoClient` is used. See :mod:`~pymongo.read_preferences`
+    :class:`MotorClient` is used. See :mod:`~pymongo.read_preferences`
     for options.
   - `write_concern` (optional): An instance of
     :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
-    default) the :attr:`write_concern` of this :class:`MongoClient` is
+    default) the :attr:`write_concern` of this :class:`MotorClient` is
     used.
 """
 
+get_default_database_doc = """
+Get the database named in the MongoDB connection URI.
+
+>>> uri = 'mongodb://host/my_database'
+>>> client = MotorClient(uri)
+>>> db = client.get_default_database()
+>>> assert db.name == 'my_database'
+>>> db = client.get_default_database('fallback_db_name')
+>>> assert db.name == 'my_database'
+>>> uri_without_database = 'mongodb://host/'
+>>> client = MotorClient(uri_without_database)
+>>> db = client.get_default_database('fallback_db_name')
+>>> assert db.name == 'fallback_db_name'
+
+
+Useful in scripts where you want to choose which database to use
+based only on the URI in a configuration file.
+
+:Parameters:
+  - `default` (optional): the database name to use if no database name
+    was provided in the URI.
+  - `codec_options` (optional): An instance of
+    :class:`~bson.codec_options.CodecOptions`. If ``None`` (the
+    default) the :attr:`codec_options` of this :class:`MotorClient` is
+    used.
+  - `read_preference` (optional): The read preference to use. If
+    ``None`` (the default) the :attr:`read_preference` of this
+    :class:`MotorClient` is used. See :mod:`~pymongo.read_preferences`
+    for options.
+  - `write_concern` (optional): An instance of
+    :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
+    default) the :attr:`write_concern` of this :class:`MotorClient` is
+    used.
+  - `read_concern` (optional): An instance of
+    :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+    default) the :attr:`read_concern` of this :class:`MotorClient` is
+    used.
+
+.. versionadded:: 2.1
+   Revived this method. Added the ``default``, ``codec_options``,
+   ``read_preference``, ``write_concern`` and ``read_concern`` parameters.
+
+.. versionchanged:: 2.0
+   Removed this method.
+"""
+
+fsync_doc = """Flush all pending writes to datafiles.
+
+Optional parameters can be passed as keyword arguments:
+  - `lock`: If True lock the server to disallow writes.
+  - `async`: If True don't block while synchronizing.
+  - `session` (optional): a
+    :class:`~pymongo.client_session.ClientSession`, created with
+    :meth:`~MotorClient.start_session`.
+
+.. note:: Starting with Python 3.7 `async` is a reserved keyword.
+  The async option to the fsync command can be passed using a
+  dictionary instead::
+
+    options = {'async': True}
+    await client.fsync(**options)
+
+.. versionchanged:: 1.2
+   Added session parameter.
+
+.. warning:: `async` and `lock` can not be used together.
+
+.. warning:: MongoDB does not support the `async` option
+             on Windows and will raise an exception on that
+             platform.
+"""
+
+current_op_doc = """
+**DEPRECATED**: Get information on operations currently running.
+
+Starting with MongoDB 3.6 this helper is obsolete. The functionality
+provided by this helper is available in MongoDB 3.6+ using the
+`$currentOp aggregation pipeline stage`_, which can be used with
+:meth:`aggregate`. Note that, while this helper can only return
+a single document limited to a 16MB result, :meth:`aggregate`
+returns a cursor avoiding that limitation.
+
+Users of MongoDB versions older than 3.6 can use the `currentOp command`_
+directly::
+
+  # MongoDB 3.2 and 3.4
+  await client.admin.command("currentOp")
+
+Or query the "inprog" virtual collection::
+
+  # MongoDB 2.6 and 3.0
+  await client.admin["$cmd.sys.inprog"].find_one()
+
+:Parameters:
+  - `include_all` (optional): if ``True`` also list currently
+    idle operations in the result
+  - `session` (optional): a
+    :class:`~pymongo.client_session.ClientSession`, created with
+    :meth:`~MotorClient.start_session`.
+
+.. versionchanged:: 2.1
+   Deprecated, use :meth:`aggregate` instead.
+
+.. versionchanged:: 1.2
+   Added session parameter.
+
+.. _$currentOp aggregation pipeline stage: https://docs.mongodb.com/manual/reference/operator/aggregation/currentOp/
+.. _currentOp command: https://docs.mongodb.com/manual/reference/command/currentOp/
+"""
+
+list_collection_names_doc = """
+Get a list of all the collection names in this database.
+
+For example, to list all non-system collections::
+
+    filter = {"name": {"$regex": r"^(?!system\\.)"}}
+    names = await db.list_collection_names(filter=filter)
+
+:Parameters:
+  - `session` (optional): a
+    :class:`~pymongo.client_session.ClientSession`, created with
+    :meth:`~MotorClient.start_session`.
+  - `filter` (optional):  A query document to filter the list of
+    collections returned from the listCollections command.
+  - `**kwargs` (optional): Optional parameters of the
+    `listCollections command
+    <https://docs.mongodb.com/manual/reference/command/listCollections/>`_
+    can be passed as keyword arguments to this method. The supported
+    options differ by server version.
+
+.. versionchanged:: 2.1
+   Added the ``filter`` and ``**kwargs`` parameters.
+
+.. versionadded:: 1.2
+"""
+
 bulk_write_doc = """Send a batch of write operations to the server.
 
 Requests are passed as a list of write operation instances imported
@@ -1054,3 +1190,88 @@ started it.
 
 .. versionadded:: 1.2
 """
+
+
+with_transaction_doc = """Executes an awaitable in a transaction.
+
+This method starts a transaction on this session, awaits ``coro``
+once, and then commits the transaction. For example::
+
+  async def coro(session):
+      orders = session.client.db.orders
+      inventory = session.client.db.inventory
+      inserted_id = await orders.insert_one(
+          {"sku": "abc123", "qty": 100}, session=session)
+      await inventory.update_one(
+          {"sku": "abc123", "qty": {"$gte": 100}},
+          {"$inc": {"qty": -100}}, session=session)
+      return inserted_id
+
+  async with await client.start_session() as session:
+      inserted_id = await session.with_transaction(coro)
+
+To pass arbitrary arguments to the ``coro``, wrap it with a
+``lambda`` like this::
+
+  async def coro(session, custom_arg, custom_kwarg=None):
+      # Transaction operations...
+
+  async with await client.start_session() as session:
+      await session.with_transaction(
+          lambda s: coro(s, "custom_arg", custom_kwarg=1))
+
+In the event of an exception, ``with_transaction`` may retry the commit
+or the entire transaction, therefore ``coro`` may be awaited
+multiple times by a single call to ``with_transaction``. Developers
+should be mindful of this possiblity when writing a ``coro`` that
+modifies application state or has any other side-effects.
+Note that even when the ``coro`` is invoked multiple times,
+``with_transaction`` ensures that the transaction will be committed
+at-most-once on the server.
+
+The ``coro`` should not attempt to start new transactions, but
+should simply run operations meant to be contained within a
+transaction. The ``coro`` should also not commit the transaction;
+this is handled automatically by ``with_transaction``. If the
+``coro`` does commit or abort the transaction without error,
+however, ``with_transaction`` will return without taking further
+action.
+
+When ``coro`` raises an exception, ``with_transaction``
+automatically aborts the current transaction. When ``coro`` or
+:meth:`~ClientSession.commit_transaction` raises an exception that
+includes the ``"TransientTransactionError"`` error label,
+``with_transaction`` starts a new transaction and re-executes
+the ``coro``.
+
+When :meth:`~ClientSession.commit_transaction` raises an exception with
+the ``"UnknownTransactionCommitResult"`` error label,
+``with_transaction`` retries the commit until the result of the
+transaction is known.
+
+This method will cease retrying after 120 seconds has elapsed. This
+timeout is not configurable and any exception raised by the
+``coro`` or by :meth:`ClientSession.commit_transaction` after the
+timeout is reached will be re-raised. Applications that desire a
+different timeout duration should not use this method.
+
+:Parameters:
+  - `coro`: The coroutine to run inside a transaction. The coroutine must
+    accept a single argument, this session. Note, under certain error
+    conditions the coroutine may be run multiple times.
+  - `read_concern` (optional): The
+    :class:`~pymongo.read_concern.ReadConcern` to use for this
+    transaction.
+  - `write_concern` (optional): The
+    :class:`~pymongo.write_concern.WriteConcern` to use for this
+    transaction.
+  - `read_preference` (optional): The read preference to use for this
+    transaction. If ``None`` (the default) the :attr:`read_preference`
+    of this :class:`Database` is used. See
+    :mod:`~pymongo.read_preferences` for options.
+
+:Returns:
+  The return value of the ``coro``.
+
+.. versionadded:: 3.9
+"""
