@@ -17,11 +17,13 @@ from __future__ import unicode_literals, absolute_import
 """GridFS implementation for Motor, an asynchronous driver for MongoDB."""
 
 import textwrap
+import warnings
 
 import gridfs
 import pymongo
 import pymongo.errors
-from gridfs import grid_file
+from gridfs import (DEFAULT_CHUNK_SIZE,
+                    grid_file)
 
 from motor.core import (AgnosticBaseCursor,
                         AgnosticCollection,
@@ -151,19 +153,17 @@ class AgnosticGridOut(object):
     metadata     = MotorGridOutProperty()
     name         = MotorGridOutProperty()
     read         = AsyncRead()
+    readable     = DelegateMethod()
     readchunk    = AsyncRead()
     readline     = AsyncRead()
     seek         = DelegateMethod()
+    seekable     = DelegateMethod()
     tell         = DelegateMethod()
     upload_date  = MotorGridOutProperty()
+    write        = DelegateMethod()
 
-    def __init__(
-        self,
-        root_collection,
-        file_id=None,
-        file_document=None,
-        delegate=None,
-    ):
+    def __init__(self, root_collection, file_id=None, file_document=None,
+                 delegate=None, session=None):
         collection_class = create_class_with_framework(
             AgnosticCollection, self._framework, self.__module__)
 
@@ -178,7 +178,8 @@ class AgnosticGridOut(object):
             self.delegate = self.__delegate_class__(
                 root_collection.delegate,
                 file_id,
-                file_document)
+                file_document,
+                session=session)
 
         self.io_loop = root_collection.get_io_loop()
 
@@ -271,19 +272,23 @@ class AgnosticGridIn(object):
     __delegate_class__ = gridfs.GridIn
 
     __getattr__  = DelegateMethod()
+    _id          = ReadOnlyProperty()
     abort        = AsyncCommand()
+    chunk_size   = ReadOnlyProperty()
     closed       = ReadOnlyProperty()
     close        = AsyncCommand()
-    write        = AsyncCommand().unwrap('MotorGridOut')
-    writelines   = AsyncCommand().unwrap('MotorGridOut')
-    _id          = ReadOnlyProperty()
-    md5          = ReadOnlyProperty()
-    filename     = ReadOnlyProperty()
-    name         = ReadOnlyProperty()
     content_type = ReadOnlyProperty()
+    filename     = ReadOnlyProperty()
     length       = ReadOnlyProperty()
-    chunk_size   = ReadOnlyProperty()
+    md5          = ReadOnlyProperty()
+    name         = ReadOnlyProperty()
+    read         = DelegateMethod()
+    readable     = DelegateMethod()
+    seekable     = DelegateMethod()
     upload_date  = ReadOnlyProperty()
+    write        = AsyncCommand().unwrap('MotorGridOut')
+    writeable    = DelegateMethod()
+    writelines   = AsyncCommand().unwrap('MotorGridOut')
     set          = AsyncCommand(attr_name='__setattr__', doc="""
 Set an arbitrary metadata attribute on the file. Stores value on the server
 as a key-value pair within the file document once the file is closed. If
@@ -299,7 +304,8 @@ Metadata set on the file appears as attributes on a
   - `value`: Value of the attribute
 """)
 
-    def __init__(self, root_collection, delegate=None, **kwargs):
+    def __init__(self, root_collection, delegate=None, session=None,
+                 disable_md5=False, **kwargs):
         """
         Class to write data to GridFS. Application developers should not
         generally need to instantiate this class - see
@@ -330,8 +336,14 @@ Metadata set on the file appears as attributes on a
             :class:`bytes`.
 
         :Parameters:
-          - `root_collection`: A :class:`~motor.MotorCollection`, the root
-             collection to write to
+          - `root_collection`: root collection to write to
+          - `session` (optional): a
+            :class:`~pymongo.client_session.ClientSession` to use for all
+            commands
+          - `disable_md5` (optional): When True, an MD5 checksum will not be
+            computed for the uploaded file. Useful in environments where
+            MD5 cannot be used for regulatory or other reasons. Defaults to
+            False.
           - `**kwargs` (optional): file level options (see above)
 
         .. versionchanged:: 0.2
@@ -352,6 +364,8 @@ Metadata set on the file appears as attributes on a
         else:
             self.delegate = self.__delegate_class__(
                 root_collection.delegate,
+                session=session,
+                disable_md5=disable_md5,
                 **kwargs)
 
     if PY35:
@@ -383,7 +397,9 @@ class AgnosticGridFSBucket(object):
     upload_from_stream           = AsyncCommand()
     upload_from_stream_with_id   = AsyncCommand()
 
-    def __init__(self, database, collection="fs", disable_md5=False):
+    def __init__(self, database, bucket_name="fs", disable_md5=False,
+                 chunk_size_bytes=DEFAULT_CHUNK_SIZE, write_concern=None,
+                 read_preference=None, collection=None):
         """Create a handle to a GridFS bucket.
 
         Raises :exc:`~pymongo.errors.ConfigurationError` if `write_concern`
@@ -406,11 +422,25 @@ class AgnosticGridFSBucket(object):
           - `disable_md5` (optional): When True, MD5 checksums will not be
             computed for uploaded files. Useful in environments where MD5
             cannot be used for regulatory or other reasons. Defaults to False.
+          - `collection` (optional): Deprecated, an alias for `bucket_name`
+            that exists solely to provide backwards compatibility.
 
+        .. versionchanged:: 2.1
+           Added support for the `bucket_name`, `chunk_size_bytes`,
+           `write_concern`, and `read_preference` parameters.
+           Deprecated the `collection` parameter which is now an alias to
+           `bucket_name` (to match the GridFSBucket class in PyMongo).
         .. versionadded:: 1.0
 
         .. mongodoc:: gridfs
         """
+        # Preserve backwards compatibility of "collection" parameter
+        if collection is not None:
+            warnings.warn('the "collection" parameter is deprecated, use '
+                          '"bucket_name" instead', DeprecationWarning,
+                          stacklevel=2)
+            bucket_name = collection
+
         db_class = create_class_with_framework(
             AgnosticDatabase, self._framework, self.__module__)
 
@@ -420,10 +450,16 @@ class AgnosticGridFSBucket(object):
                     self.__class__, database))
 
         self.io_loop = database.get_io_loop()
-        self.collection = database[collection]
+        self.collection = database.get_collection(
+            bucket_name,
+            write_concern=write_concern,
+            read_preference=read_preference)
         self.delegate = self.__delegate_class__(
             database.delegate,
-            collection,
+            bucket_name,
+            chunk_size_bytes=chunk_size_bytes,
+            write_concern=write_concern,
+            read_preference=read_preference,
             disable_md5=disable_md5)
 
     def get_io_loop(self):
