@@ -22,6 +22,7 @@ DO NOT USE THIS MODULE.
 """
 
 import inspect
+import unittest
 from tornado.ioloop import IOLoop
 
 import motor
@@ -32,7 +33,12 @@ from motor.metaprogramming import MotorAttributeFactory
 
 # Make e.g. "from pymongo.errors import AutoReconnect" work. Note that
 # importing * won't pick up underscore-prefixed attrs.
+from gridfs import *
 from gridfs.errors import *
+from gridfs.grid_file import (DEFAULT_CHUNK_SIZE,
+                              _SEEK_CUR,
+                              _SEEK_END,
+                              _clear_entity_type_registry)
 from pymongo import *
 from pymongo import (collation,
                      compression_support,
@@ -54,9 +60,12 @@ from pymongo.client_session import TransactionOptions
 from pymongo.collation import *
 from pymongo.common import *
 from pymongo.common import _UUID_REPRESENTATIONS, _MAX_END_SESSIONS
-from pymongo.compression_support import _HAVE_SNAPPY, _HAVE_ZLIB
+from pymongo.compression_support import _HAVE_SNAPPY, _HAVE_ZLIB, _HAVE_ZSTD
 from pymongo.cursor import *
 from pymongo.cursor import _QUERY_OPTIONS
+from pymongo.encryption import *
+from pymongo.encryption_options import *
+from pymongo.encryption_options import _HAVE_PYMONGOCRYPT
 from pymongo.errors import *
 from pymongo.message import (_COMMAND_OVERHEAD,
                              _CursorAddress,
@@ -68,7 +77,7 @@ from pymongo.monitoring import _LISTENERS, _Listeners, _SENSITIVE_COMMANDS
 from pymongo.monotonic import time
 from pymongo.operations import *
 from pymongo.pool import *
-from pymongo.pool import _METADATA
+from pymongo.pool import _METADATA, _PoolClosedError
 from pymongo.periodic_executor import *
 from pymongo.periodic_executor import _EXECUTORS
 from pymongo.read_concern import *
@@ -85,12 +94,11 @@ from pymongo.son_manipulator import *
 from pymongo.topology import *
 from pymongo.topology_description import *
 from pymongo.uri_parser import *
-from pymongo.uri_parser import _partition, _rpartition, _HAVE_DNSPYTHON
+from pymongo.uri_parser import _HAVE_DNSPYTHON
 from pymongo.write_concern import *
 from pymongo import auth
 from pymongo.auth import *
 from pymongo.auth import _password_digest
-from gridfs.grid_file import DEFAULT_CHUNK_SIZE, _SEEK_CUR, _SEEK_END
 
 from pymongo import GEOSPHERE, HASHED
 from pymongo.pool import SocketInfo, Pool
@@ -130,6 +138,8 @@ def wrap_synchro(fn):
             client = MongoClient(delegate=motor_obj.client)
             return Database(client, motor_obj.name, delegate=motor_obj)
         if isinstance(motor_obj, motor.motor_tornado.MotorChangeStream):
+            # Send the initial aggregate as PyMongo expects.
+            motor_obj._lazy_init()
             return ChangeStream(motor_obj)
         if isinstance(motor_obj, motor.motor_tornado.MotorLatentCommandCursor):
             return CommandCursor(motor_obj)
@@ -276,7 +286,15 @@ class SynchroMeta(type):
         return new_class
 
 
-class Synchro(object):
+def with_metaclass(metaclass, *bases):
+    """Python 2/3 compatible metaclass helper."""
+    class _metaclass(metaclass):
+        def __new__(mcls, name, _bases, attrs):
+            return metaclass(name, bases, attrs)
+    return type.__new__(_metaclass, str('dummy'), (), {})
+
+
+class Synchro(with_metaclass(SynchroMeta)):
     """
     Wraps a MotorClient, MotorDatabase, MotorCollection, etc. and
     makes it act like the synchronous pymongo equivalent
@@ -285,8 +303,17 @@ class Synchro(object):
     __delegate_class__ = None
 
     def __cmp__(self, other):
+        """Implements == and != on Python 2."""
         return cmp(self.delegate, other.delegate)
 
+    def __eq__(self, other):
+        """Implements == and != on Python 3."""
+        if (isinstance(other, self.__class__)
+                and hasattr(self, 'delegate')
+                and hasattr(other, 'delegate')):
+            return self.delegate == other.delegate
+        return NotImplemented
+
     def synchronize(self, async_method):
         """
         @param async_method: Bound method of a MotorClient, MotorDatabase, etc.
@@ -308,11 +335,11 @@ class MongoClient(Synchro):
     HOST = 'localhost'
     PORT = 27017
 
-    _cache_credentials = SynchroProperty()
     get_database = WrapOutgoing()
     max_pool_size = SynchroProperty()
     max_write_batch_size = SynchroProperty()
     start_session = Sync()
+    watch = WrapOutgoing()
 
     def __init__(self, host=None, port=None, *args, **kwargs):
         # So that TestClient.test_constants and test_types work.
@@ -345,8 +372,11 @@ class MongoClient(Synchro):
     def __getitem__(self, name):
         return Database(self, name, delegate=self.delegate[name])
 
+    # For PyMongo tests that access client internals.
     _MongoClient__all_credentials = SynchroProperty()
     _MongoClient__options         = SynchroProperty()
+    _cache_credentials            = SynchroProperty()
+    _close_cursor_now             = SynchroProperty()
     _get_topology                 = SynchroProperty()
     _topology                     = SynchroProperty()
     _kill_cursors_executor        = SynchroProperty()
@@ -360,7 +390,12 @@ class _SynchroTransactionContext(Synchro):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.synchronize(self.delegate._session.end_session)()
+        motor_session = self.delegate._session
+        if motor_session.in_transaction:
+            if exc_val is None:
+                self.synchronize(motor_session.commit_transaction)()
+            else:
+                self.synchronize(motor_session.abort_transaction)()
 
 
 class ClientSession(Synchro):
@@ -378,16 +413,20 @@ class ClientSession(Synchro):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.synchronize(self.delegate.end_session)
 
+    # For PyMongo tests that access session internals.
     _client              = SynchroProperty()
-    _in_transaction      = SynchroProperty()
+    _pinned_address      = SynchroProperty()
     _server_session      = SynchroProperty()
+    _transaction         = SynchroProperty()
     _transaction_id      = SynchroProperty()
     _txn_read_preference = SynchroProperty()
 
 
 class Database(Synchro):
     __delegate_class__ = motor.MotorDatabase
+
     get_collection     = WrapOutgoing()
+    watch              = WrapOutgoing()
 
     def __init__(self, client, name, **kwargs):
         assert isinstance(client, MongoClient), (
@@ -402,6 +441,13 @@ class Database(Synchro):
             "synchro.Database delegate must be MotorDatabase, not "
             " %s" % repr(self.delegate))
 
+    def aggregate(self, *args, **kwargs):
+        # Motor does no I/O initially in aggregate() but PyMongo does.
+        func = wrap_synchro(unwrap_synchro(self.delegate.aggregate))
+        cursor = func(*args, **kwargs)
+        self.synchronize(cursor.delegate._get_more)()
+        return cursor
+
     @property
     def client(self):
         return self._client
@@ -462,12 +508,25 @@ class Collection(Synchro):
         return Collection(self.database, fullname,
                           delegate=self.delegate[name])
 
+    def count(self, *args, **kwargs):
+        raise unittest.SkipTest('count() is not supported in Motor')
+
+    def group(self, *args, **kwargs):
+        raise unittest.SkipTest('group() is not supported in Motor')
+
 
 class ChangeStream(Synchro):
     __delegate_class__ = motor.motor_tornado.MotorChangeStream
 
-    next = Sync('next')
-    close = Sync('close')
+    _next    = Sync('next')
+    try_next = Sync('try_next')
+    close    = Sync('close')
+
+    def next(self):
+        try:
+            return self._next()
+        except StopAsyncIteration:
+            raise StopIteration()
 
     def __init__(self, motor_change_stream):
         self.delegate = motor_change_stream
@@ -478,6 +537,24 @@ class ChangeStream(Synchro):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
+    def __iter__(self):
+        return self
+
+    __next__ = next
+
+    # For PyMongo tests that access change stream internals.
+
+    @property
+    def _cursor(self):
+        raise unittest.SkipTest('test accesses internal _cursor field')
+
+    _batch_size        = SynchroProperty()
+    _client            = SynchroProperty()
+    _full_document     = SynchroProperty()
+    _max_await_time_ms = SynchroProperty()
+    _pipeline          = SynchroProperty()
+    _target            = SynchroProperty()
+
 
 class Cursor(Synchro):
     __delegate_class__ = motor.motor_tornado.MotorCursor
@@ -515,6 +592,8 @@ class Cursor(Synchro):
 
         raise StopIteration
 
+    __next__ = next
+
     def __getitem__(self, index):
         if isinstance(index, slice):
             return Cursor(self.delegate[index])
@@ -560,7 +639,7 @@ class GridOutCursor(Cursor):
             raise TypeError(
                 "Expected MotorGridOutCursor, got %r" % delegate)
 
-        self.delegate = delegate
+        super(GridOutCursor, self).__init__(delegate)
 
     def next(self):
         motor_grid_out = super(GridOutCursor, self).next()
@@ -582,18 +661,15 @@ class BulkOperationBuilder(object):
 class GridFSBucket(Synchro):
     __delegate_class__ = motor.MotorGridFSBucket
 
-    def __init__(self, database, bucket_name='fs', disable_md5=False):
+    find = WrapOutgoing()
+
+    def __init__(self, database, *args, **kwargs):
         if not isinstance(database, Database):
             raise TypeError(
                 "Expected Database, got %s" % repr(database))
 
         self.delegate = motor.MotorGridFSBucket(
-            database.delegate, bucket_name, disable_md5)
-
-    def find(self, *args, **kwargs):
-        motor_method = self.delegate.find
-        unwrapping_method = wrap_synchro(unwrap_synchro(motor_method))
-        return unwrapping_method(*args, **kwargs)
+            database.delegate, *args, **kwargs)
 
 
 class GridIn(Synchro):
@@ -616,6 +692,12 @@ class GridIn(Synchro):
     def __getattr__(self, item):
         return getattr(self.delegate, item)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
 
 class SynchroGridOutProperty(object):
     def __init__(self, name):
@@ -643,7 +725,7 @@ class GridOut(Synchro):
 
     def __init__(
             self, root_collection, file_id=None, file_document=None,
-            delegate=None):
+            session=None, delegate=None):
         """Can be created with collection and kwargs like a PyMongo GridOut,
         or with a 'delegate' keyword arg, where delegate is a MotorGridOut.
         """
@@ -655,7 +737,8 @@ class GridOut(Synchro):
                     "Expected Collection, got %s" % repr(root_collection))
 
             self.delegate = motor.MotorGridOut(
-                root_collection.delegate, file_id, file_document)
+                root_collection.delegate, file_id, file_document,
+                session=session)
 
     def __getattr__(self, item):
         self.synchronize(self.delegate.open)()
@@ -671,19 +754,23 @@ class GridOut(Synchro):
 
         super(GridOut, self).__setattr__(key, value)
 
+    def __enter__(self):
+        return self
 
-class TimeModule(object):
-    """Fake time module so time.sleep() lets other tasks run on the IOLoop.
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
 
-    See e.g. test_schedule_refresh() in test_replica_set_client.py.
-    """
-    def __getattr__(self, item):
-        def sleep(seconds):
-            loop = IOLoop.current()
-            loop.add_timeout(time.time() + seconds, loop.stop)
-            loop.start()
-
-        if item == 'sleep':
-            return sleep
+    def __next__(self):
+        if sys.version_info >= (3, 5):
+            try:
+                return self.synchronize(self.delegate.__anext__)()
+            except StopAsyncIteration:
+                raise StopIteration()
         else:
-            return getattr(time, item)
+            chunk = self.readchunk()
+            if chunk:
+                return chunk
+            raise StopIteration()
+
+    def __iter__(self):
+        return self
