@@ -53,14 +53,17 @@ excluded_modules = [
     # Accesses PyMongo internals. Tested directly in Motor.
     'test.test_session',
 
-    # We test Synchro in Python 2.7, but Motor's change streams need Python 3.
-    'test.test_change_stream',
-
     # Deprecated in PyMongo, removed in Motor 2.0.
     'test.test_gridfs',
     'test.test_son_manipulator',
 ]
 
+if sys.version_info[:2] < (3, 5):
+    excluded_modules.extend([
+        # Motor's change streams need Python 3.5.
+        'test.test_change_stream',
+    ])
+
 excluded_tests = [
     # Motor's reprs aren't the same as PyMongo's.
     '*.test_repr',
@@ -133,6 +136,7 @@ excluded_tests = [
 
     # Motor is correct here, it's just unreliable on slow CI servers.
     'TestReplicaSetClient.test_timeout_does_not_mark_member_down',
+    'TestCMAP.test_cmap_wait_queue_timeout_must_aggressively_timeout_threads_enqueued_longer_than_waitQueueTimeoutMS',
 
     # Accesses PyMongo internals.
     'TestClient.test_close_kills_cursors',
@@ -142,14 +146,14 @@ excluded_tests = [
     'TestCommandMonitoring.test_get_more_failure',
     'TestCommandMonitoring.test_sensitive_commands',
     'TestCursor.test_close_kills_cursor_synchronously',
+    'TestCursor.test_delete_not_initialized',
     'TestGridFile.test_grid_out_cursor_options',
+    'TestGridFile.test_survive_cursor_not_found',
     'TestMaxStaleness.test_last_write_date',
     'TestMaxStaleness.test_last_write_date_absent',
-    'TestMonitor.test_atexit_hook',
     'TestReplicaSetClient.test_kill_cursor_explicit_primary',
     'TestReplicaSetClient.test_kill_cursor_explicit_secondary',
     'TestSelections.test_bool',
-    'TestTransactions.transaction_test_debug',
 
     # Deprecated in PyMongo, removed in Motor 2.0.
     'TestDatabase.test_collection_names',
@@ -163,12 +167,13 @@ excluded_tests = [
     'TestCommandMonitoring.test_legacy_insert_many',
     'TestCommandMonitoring.test_legacy_writes',
     'TestClient.test_database_names',
+    'TestClient.test_is_locked_does_not_raise_warning',
+    'TestCollectionWCustomType.test_find_and_modify_w_custom_type_decoder',
 
     # Tests that use "count", deprecated in PyMongo, removed in Motor 2.0.
     '*.test_command_monitoring_command_A_failed_command_event',
     '*.test_command_monitoring_command_A_successful_command',
     '*.test_command_monitoring_command_A_successful_command_with_a_non-primary_read_preference',
-    '*.test_read_count-collation_Deprecated_count_with_collation',
     '*.test_read_count_Deprecated_count_with_a_filter',
     '*.test_read_count_Deprecated_count_without_a_filter',
     'TestBinary.test_uuid_queries',
@@ -179,12 +184,12 @@ excluded_tests = [
     'TestCursor.test_count_with_hint',
     'TestCursor.test_where',
     'TestGridfs.test_gridfs_find',
-    'TestTransactions.test_transactions_reads_count',
 
     # Tests that use "authenticate" or "logoout", removed in Motor 2.0.
     'TestSASLPlain.test_sasl_plain_bad_credentials',
     'TestSCRAM.test_scram',
     'TestSCRAMSHA1.test_scram_sha1',
+    'TestThreadedAuth.*',
 
     # Uses "collection_names", deprecated in PyMongo, removed in Motor 2.0.
     'TestSingleSlaveOk.test_reads_from_secondary',
@@ -192,8 +197,28 @@ excluded_tests = [
     # Slow.
     'TestDatabase.test_collection_names_single_socket',
     'TestDatabase.test_list_collection_names',
+
+    # MOTOR-425 these tests fail with duplicate key errors.
+    'TestClusterChangeStreamsWCustomTypes.*',
+    'TestCollectionChangeStreamsWCustomTypes.*',
+    'TestDatabaseChangeStreamsWCustomTypes.*',
+
+    # Tests that use warnings.catch_warnings which don't show up in Motor
+    'TestCursor.test_min_max_without_hint',
+
+    # TODO: MOTOR-280
+    'TestTransactionsConvenientAPI.*',
 ]
 
+if sys.version_info[:2] >= (3, 5):
+    excluded_tests.extend([
+        # Motor's change streams need Python 3.5 to support async iteration but
+        # these change streams tests spawn threads which don't work without an
+        # IO loop.
+        '*.test_next_blocks',
+        '*.test_aggregate_cursor_blocks',
+    ])
+
 
 excluded_modules_matched = set()
 excluded_tests_matched = set()
@@ -236,17 +261,27 @@ class SynchroNosePlugin(Plugin):
         # them as unittests.
         if fn.__name__ in ('test_cases',
                            'create_test',
+                           'create_tests',
                            'create_connection_string_test',
                            'create_document_test',
                            'create_selection_tests',
                            ):
             return False
 
+    def wantClass(self, cls):
+        # PyMongo's test generator classes run at import time; tell Nose not
+        # to run them as unittests.
+        if cls.__name__ in ('TestCreator',):
+            return False
+
     def wantMethod(self, method):
         # Run standard Nose checks on name, like "does it start with test_"?
         if not self.selector.matches(method.__name__):
             return False
 
+        if method.__name__ in ('run_test_ops',):
+            return False
+
         for excluded_name in excluded_tests:
             if PY3:
                 classname = method.__self__.__class__.__name__
@@ -268,36 +303,101 @@ class SynchroNosePlugin(Plugin):
         return True
 
 
-# So that e.g. 'from pymongo.mongo_client import MongoClient' gets the
-# Synchro MongoClient, not the real one.
-class SynchroModuleFinder(object):
-    def find_module(self, fullname, path=None):
-        parts = fullname.split('.')
-        if parts[-1] in ('gridfs', 'pymongo'):
-            return SynchroModuleLoader(path)
-        elif len(parts) >= 2 and parts[-2] in ('gridfs', 'pymongo'):
-            return SynchroModuleLoader(path)
+if sys.version_info[0] < 3:
+    # So that e.g. 'from pymongo.mongo_client import MongoClient' gets the
+    # Synchro MongoClient, not the real one.
+    class SynchroModuleFinder(object):
+        def find_module(self, fullname, path=None):
+            parts = fullname.split('.')
+            if parts[-1] in ('gridfs', 'pymongo'):
+                # E.g. "import pymongo"
+                return SynchroModuleLoader(path)
+            elif len(parts) >= 2 and parts[-2] in ('gridfs', 'pymongo'):
+                # E.g. "import pymongo.mongo_client"
+                return SynchroModuleLoader(path)
 
-        # Let regular module search continue.
-        return None
+            # Let regular module search continue.
+            return None
 
 
-class SynchroModuleLoader(object):
-    def __init__(self, path):
-        self.path = path
+    class SynchroModuleLoader(object):
+        def __init__(self, path):
+            self.path = path
 
-    def load_module(self, fullname):
-        return synchro
+        def load_module(self, fullname):
+            return synchro
+else:
+    import importlib
+    import importlib.abc
+    import importlib.machinery
 
+    class SynchroModuleFinder(importlib.abc.MetaPathFinder):
+        def __init__(self):
+            self._loader = SynchroModuleLoader()
+
+        def find_spec(self, fullname, path, target=None):
+            if self._loader.patch_spec(fullname):
+                return importlib.machinery.ModuleSpec(fullname, self._loader)
+
+            # Let regular module search continue.
+            return None
+
+
+    class SynchroModuleLoader(importlib.abc.Loader):
+        def patch_spec(self, fullname):
+            parts = fullname.split('.')
+            if parts[-1] in ('gridfs', 'pymongo'):
+                # E.g. "import pymongo"
+                return True
+            elif len(parts) >= 2 and parts[-2] in ('gridfs', 'pymongo'):
+                # E.g. "import pymongo.mongo_client"
+                return True
+
+            return False
+
+        def exec_module(self, module):
+            pass
+
+        def create_module(self, spec):
+            if self.patch_spec(spec.name):
+                return synchro
+
+            # Let regular module search continue.
+            return None
 
 if __name__ == '__main__':
+    try:
+        # Enable the fault handler to dump the traceback of each running
+        # thread
+        # after a segfault.
+        import faulthandler
+
+        faulthandler.enable()
+        # Dump the tracebacks of all threads after 25 minutes.
+        if hasattr(faulthandler, 'dump_traceback_later'):
+            faulthandler.dump_traceback_later(25 * 60)
+    except ImportError:
+        pass
+
     # Monkey-patch all pymongo's unittests so they think Synchro is the
     # real PyMongo.
     sys.meta_path[0:0] = [SynchroModuleFinder()]
-
-    # Ensure time.sleep() acts as PyMongo's tests expect: background tasks
-    # can run to completion while foreground pauses.
-    sys.modules['time'] = synchro.TimeModule()
+    # Delete the cached pymongo/gridfs modules so that SynchroModuleFinder will
+    # be invoked in Python 3, see
+    # https://docs.python.org/3/reference/import.html#import-hooks
+    for n in ['pymongo',
+              'pymongo.collection',
+              'pymongo.client_session',
+              'pymongo.command_cursor',
+              'pymongo.change_stream',
+              'pymongo.cursor',
+              'pymongo.mongo_client',
+              'pymongo.database',
+              'pymongo.mongo_replica_set_client',
+              'gridfs',
+              'gridfs.grid_file',
+              'pymongo.encryption']:
+        sys.modules.pop(n)
 
     if '--check-exclude-patterns' in sys.argv:
         check_exclude_patterns = True
