@@ -22,9 +22,9 @@ import unittest
 import bson
 from bson import CodecOptions
 from bson.binary import JAVA_LEGACY
-from bson.objectid import ObjectId
 from pymongo import ReadPreference, WriteConcern
-from pymongo.errors import DuplicateKeyError, OperationFailure
+from pymongo.errors import BulkWriteError, DuplicateKeyError, OperationFailure
+from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import Secondary
 
 from motor import motor_asyncio
@@ -90,13 +90,11 @@ class TestAsyncIOCollection(AsyncIOTestCase):
     @asyncio_test
     def test_update(self):
         yield from self.collection.insert_one({'_id': 1})
-        result = yield from self.collection.update(
+        result = yield from self.collection.update_one(
             {'_id': 1}, {'$set': {'foo': 'bar'}})
 
-        self.assertEqual(1, result['ok'])
-        self.assertEqual(True, result['updatedExisting'])
-        self.assertEqual(1, result['n'])
-        self.assertEqual(None, result.get('err'))
+        self.assertIsNone(result.upserted_id)
+        self.assertEqual(1, result.modified_count)
 
     @ignore_deprecations
     @asyncio_test
@@ -108,7 +106,7 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         try:
             yield from coll.insert_many([{'s': 1}, {'s': 2}])
             with self.assertRaises(DuplicateKeyError):
-                yield from coll.update({'s': 2}, {'$set': {'s': 1}})
+                yield from coll.update_one({'s': 2}, {'$set': {'s': 1}})
 
         finally:
             yield from coll.drop()
@@ -126,8 +124,8 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         yield from collection.insert_one({'_id': 2})
 
         # Violate a unique index in one of many updates, handle error.
-        with self.assertRaises(DuplicateKeyError):
-            yield from collection.insert([
+        with self.assertRaises(BulkWriteError):
+            yield from collection.insert_many([
                 {'_id': 1},
                 {'_id': 2},  # Already exists
                 {'_id': 3}])
@@ -137,36 +135,6 @@ class TestAsyncIOCollection(AsyncIOTestCase):
             set([1, 2]),
             set((yield from collection.distinct('_id'))))
 
-    @ignore_deprecations
-    @asyncio_test
-    def test_save_with_id(self):
-        # save() returns the _id, in this case 5.
-        self.assertEqual(
-            5,
-            (yield from self.collection.save({'_id': 5})))
-
-    @ignore_deprecations
-    @asyncio_test
-    def test_save_without_id(self):
-        collection = self.collection
-        result = yield from collection.save({'fiddle': 'faddle'})
-
-        # save() returns the new _id
-        self.assertTrue(isinstance(result, ObjectId))
-
-    @ignore_deprecations
-    @asyncio_test
-    def test_save_bad(self):
-        coll = self.db.unique_collection
-        yield from coll.ensure_index('s', unique=True)
-        yield from coll.save({'s': 1})
-
-        try:
-            with self.assertRaises(DuplicateKeyError):
-                yield from coll.save({'s': 1})
-        finally:
-            yield from coll.drop()
-
     @asyncio_test
     def test_delete_one(self):
         # Remove a document twice, check that we get a success responses
@@ -190,74 +158,25 @@ class TestAsyncIOCollection(AsyncIOTestCase):
     @asyncio_test
     def test_unacknowledged_insert(self):
         coll = self.db.test_unacknowledged_insert
-        coll.with_options(write_concern=WriteConcern(0)).insert_one({'_id': 1})
+        yield from coll.with_options(write_concern=WriteConcern(0)).insert_one(
+            {'_id': 1})
 
         # The insert is eventually executed.
-        while not (yield from coll.count()):
+        while not (yield from coll.count_documents({})):
             yield from asyncio.sleep(0.1, loop=self.loop)
 
-        # DuplicateKeyError not raised.
-        future = coll.insert({'_id': 1})
-        yield from coll.insert({'_id': 1}, w=0)
-
-        with self.assertRaises(DuplicateKeyError):
-            yield from future
-
-    @ignore_deprecations
-    @asyncio_test
-    def test_unacknowledged_save(self):
-        # Test that unsafe saves with no callback still work
-        collection_name = 'test_unacknowledged_save'
-        coll = self.db[collection_name]
-        future = coll.save({'_id': 201}, w=0)
-
-        while not (yield from coll.find_one({'_id': 201})):
-            yield from asyncio.sleep(0.1, loop=self.loop)
-
-        # DuplicateKeyError not raised
-        coll.save({'_id': 201})
-        yield from coll.save({'_id': 201}, w=0)
-
-        # Clean up.
-        yield from future
-        coll.database.client.close()
-
     @ignore_deprecations
     @asyncio_test
     def test_unacknowledged_update(self):
-        # Test that unsafe updates with no callback still work
         coll = self.collection
 
         yield from coll.insert_one({'_id': 1})
-        coll.update({'_id': 1}, {'$set': {'a': 1}}, w=0)
+        yield from coll.with_options(write_concern=WriteConcern(0)).update_one(
+            {'_id': 1}, {'$set': {'a': 1}})
 
         while not (yield from coll.find_one({'a': 1})):
             yield from asyncio.sleep(0.1, loop=self.loop)
 
-        coll.database.client.close()
-
-    @asyncio_test(timeout=30)
-    def test_nested_callbacks(self):
-        results = [0]
-        future = asyncio.Future(loop=self.loop)
-        yield from self.collection.delete_many({})
-        yield from self.collection.insert_one({'_id': 1})
-
-        def callback(result, error):
-            if error:
-                future.set_exception(error)
-            elif result:
-                results[0] += 1
-                if results[0] < 1000:
-                    self.collection.find({'_id': 1}).each(callback)
-                else:
-                    future.set_result(None)
-
-        self.collection.find({'_id': 1}).each(callback)
-
-        yield from future
-        self.assertEqual(1000, results[0])
-
     @asyncio_test
     def test_map_reduce(self):
         # Count number of documents with even and odd _id
@@ -317,18 +236,6 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         index_info = yield from test_collection.index_information()
         self.assertEqual([('foo', 1)], index_info[idx_name]['key'])
 
-        # Ensure the same index, test that callback is executed
-        result = yield from test_collection.ensure_index([('foo', 1)])
-        self.assertEqual('foo_1', result)
-        result2 = yield from test_collection.ensure_index([('foo', 1)])
-        self.assertEqual(None, result2)
-
-        # Ensure an index that doesn't exist, test it's created
-        yield from test_collection.ensure_index([('bar', 1)])
-        index_info = yield from test_collection.index_information()
-        self.assertTrue(any([
-            info['key'] == [('bar', 1)] for info in index_info.values()]))
-
         # Don't test drop_index or drop_indexes -- Synchro tests them
 
     @asyncio.coroutine
@@ -369,38 +276,6 @@ class TestAsyncIOCollection(AsyncIOTestCase):
             self.assertTrue('_unpack_response' in formatted
                             or '_check_command_response' in formatted)
 
-    @asyncio_test(timeout=30)
-    def test_parallel_scan(self):
-        yield from skip_if_mongos(self.cx)
-
-        collection = self.collection.with_options(
-            write_concern=WriteConcern(test.env.w))
-
-        # Enough documents that each cursor requires multiple batches.
-        yield from collection.delete_many({})
-        yield from collection.insert_many(({'_id': i} for i in range(8000)))
-
-        if test.env.is_replica_set:
-            # Test that getMore messages are sent to the right server.
-            client = self.asyncio_rsc(read_preference=Secondary())
-            collection = client.motor_test.test_collection
-
-        docs = []
-
-        @asyncio.coroutine
-        def f(cursor):
-            self.assertTrue(isinstance(cursor, AsyncIOMotorCommandCursor))
-
-            while (yield from cursor.fetch_next):
-                docs.append(cursor.next_object())
-
-        cursors = yield from collection.parallel_scan(3)
-        yield from asyncio.wait(
-            [f(cursor) for cursor in cursors],
-            loop=self.loop)
-
-        self.assertEqual(len(docs), (yield from collection.count()))
-
     def test_with_options(self):
         coll = self.db.test
         codec_options = CodecOptions(
@@ -421,6 +296,31 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         self.assertEqual(coll.codec_options, coll2.codec_options)
         self.assertEqual(coll.write_concern, coll2.write_concern)
 
+    def test_sub_collection(self):
+        # Verify that a collection with a dotted name inherits options from its
+        # parent collection.
+        write_concern = WriteConcern(w=2, j=True)
+        read_concern = ReadConcern("majority")
+        read_preference = Secondary([{"dc": "sf"}])
+        codec_options = CodecOptions(
+            tz_aware=True, uuid_representation=JAVA_LEGACY)
+
+        coll1 = self.db.get_collection(
+            'test',
+            write_concern=write_concern,
+            read_concern=read_concern,
+            read_preference=read_preference,
+            codec_options=codec_options)
+
+        coll2 = coll1.subcollection
+        coll3 = coll1['subcollection']
+
+        for c in [coll1, coll2, coll3]:
+            self.assertEqual(write_concern, c.write_concern)
+            self.assertEqual(read_concern, c.read_concern)
+            self.assertEqual(read_preference, c.read_preference)
+            self.assertEqual(codec_options, c.codec_options)
+
 
 if __name__ == '__main__':
     unittest.main()
