@@ -15,6 +15,7 @@
 """MongoDB documentation examples with Motor and asyncio."""
 
 import asyncio
+import datetime
 from io import StringIO
 from unittest.mock import patch
 
@@ -22,6 +23,7 @@ import pymongo
 from pymongo import WriteConcern
 from pymongo.errors import ConnectionFailure, OperationFailure
 from pymongo.read_concern import ReadConcern
+from pymongo.read_preferences import ReadPreference
 
 from test import env
 from test.asyncio_tests import AsyncIOTestCase, asyncio_test
@@ -676,8 +678,8 @@ class TestExamples(AsyncIOTestCase):
 
         async def insert_docs():
             while not done:
-                await db.inventory.insert_one({})
-                await asyncio.sleep(0.1)
+                await db.inventory.insert_one({"username": "alice"})
+                await db.inventory.delete_one({"username": "alice"})
 
         task = asyncio.ensure_future(insert_docs())
 
@@ -693,11 +695,19 @@ class TestExamples(AsyncIOTestCase):
             # End Changestream Example 2
 
             # Start Changestream Example 3
-            resume_token = document.get("_id")
+            resume_token = cursor.resume_token
             cursor = db.inventory.watch(resume_after=resume_token)
             document = await cursor.next()
-
             # End Changestream Example 3
+
+            # Start Changestream Example 4
+            pipeline = [
+                {'$match': {'fullDocument.username': 'alice'}},
+                {'$addFields': {'newField': 'this is an added field!'}}
+            ]
+            cursor = db.inventory.watch(pipeline=pipeline)
+            document = await cursor.next()
+            # End Changestream Example 4
         finally:
             done = True
             await task
@@ -832,6 +842,7 @@ class TestExamples(AsyncIOTestCase):
         self.addCleanup(env.sync_cx.drop_database, "hr")
         self.addCleanup(env.sync_cx.drop_database, "reporting")
 
+        client = self.cx
         employees = self.cx.hr.employees
         events = self.cx.reporting.events
         await employees.insert_one({"employee": 3, "status": "Active"})
@@ -875,10 +886,224 @@ class TestExamples(AsyncIOTestCase):
 
         # Test the example.
         with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
-            async with await self.cx.start_session() as s:
+            async with await client.start_session() as s:
                 await update_employee_info(s)
 
         employee = await employees.find_one({"employee": 3})
         self.assertIsNotNone(employee)
         self.assertEqual(employee['status'], 'Inactive')
         self.assertIn("Transaction committed", mock_stdout.getvalue())
+
+        # Start Transactions Retry Example 1
+        async def run_transaction_with_retry(txn_coro, session):
+            while True:
+                try:
+                    await txn_coro(session)  # performs transaction
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    print("Transaction aborted. Caught exception during "
+                          "transaction.")
+
+                    # If transient error, retry the whole transaction
+                    if exc.has_error_label("TransientTransactionError"):
+                        print("TransientTransactionError, retrying"
+                              "transaction ...")
+                        continue
+                    else:
+                        raise
+        # End Transactions Retry Example 1
+
+        # Test the example.
+        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
+            async with await client.start_session() as s:
+                await run_transaction_with_retry(update_employee_info, s)
+
+        employee = await employees.find_one({"employee": 3})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Inactive')
+        self.assertIn("Transaction committed", mock_stdout.getvalue())
+
+        # Start Transactions Retry Example 2
+        async def commit_with_retry(session):
+            while True:
+                try:
+                    # Commit uses write concern set at transaction start.
+                    await session.commit_transaction()
+                    print("Transaction committed.")
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    # Can retry commit
+                    if exc.has_error_label("UnknownTransactionCommitResult"):
+                        print("UnknownTransactionCommitResult, retrying "
+                              "commit operation ...")
+                        continue
+                    else:
+                        print("Error during commit ...")
+                        raise
+        # End Transactions Retry Example 2
+
+        # Test commit_with_retry from the previous examples
+        async def _insert_employee_retry_commit(session):
+            async with session.start_transaction():
+                await employees.insert_one(
+                    {"employee": 4, "status": "Active"},
+                    session=session)
+                await events.insert_one(
+                    {"employee": 4, "status": {"new": "Active", "old": None}},
+                    session=session)
+
+                await commit_with_retry(session)
+
+        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
+            async with await client.start_session() as s:
+                await run_transaction_with_retry(
+                    _insert_employee_retry_commit, s)
+
+        employee = await employees.find_one({"employee": 4})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Active')
+        self.assertIn("Transaction committed", mock_stdout.getvalue())
+
+        # Start Transactions Retry Example 3
+
+        async def run_transaction_with_retry(txn_coro, session):
+            while True:
+                try:
+                    await txn_coro(session)  # performs transaction
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    # If transient error, retry the whole transaction
+                    if exc.has_error_label("TransientTransactionError"):
+                        print("TransientTransactionError, retrying "
+                              "transaction ...")
+                        continue
+                    else:
+                        raise
+
+        async def commit_with_retry(session):
+            while True:
+                try:
+                    # Commit uses write concern set at transaction start.
+                    await session.commit_transaction()
+                    print("Transaction committed.")
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    # Can retry commit
+                    if exc.has_error_label("UnknownTransactionCommitResult"):
+                        print("UnknownTransactionCommitResult, retrying "
+                              "commit operation ...")
+                        continue
+                    else:
+                        print("Error during commit ...")
+                        raise
+
+        # Updates two collections in a transactions
+
+        async def update_employee_info(session):
+            employees_coll = session.client.hr.employees
+            events_coll = session.client.reporting.events
+
+            async with session.start_transaction(
+                    read_concern=ReadConcern("snapshot"),
+                    write_concern=WriteConcern(w="majority"),
+                    read_preference=ReadPreference.PRIMARY):
+                await employees_coll.update_one(
+                    {"employee": 3}, {"$set": {"status": "Inactive"}},
+                    session=session)
+                await events_coll.insert_one(
+                    {"employee": 3, "status": {
+                        "new": "Inactive", "old": "Active"}},
+                    session=session)
+
+                await commit_with_retry(session)
+
+        # Start a session.
+        async with await client.start_session() as session:
+            try:
+                await run_transaction_with_retry(update_employee_info, session)
+            except Exception as exc:
+                # Do something with error.
+                raise
+
+        # End Transactions Retry Example 3
+
+        employee = await employees.find_one({"employee": 3})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Inactive')
+
+        AsyncIOMotorClient = lambda _: self.cx
+        uriString = None
+
+        # Start Transactions withTxn API Example 1
+
+        # For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
+        # uriString = 'mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl'
+        # For a sharded cluster, connect to the mongos instances; e.g.
+        # uriString = 'mongodb://mongos0.example.com:27017,mongos1.example.com:27017/'
+
+        client = AsyncIOMotorClient(uriString)
+        wc_majority = WriteConcern("majority", wtimeout=1000)
+
+        # Prereq: Create collections. CRUD operations in transactions must be on existing collections.
+        await client.get_database(
+            "mydb1", write_concern=wc_majority).foo.insert_one({'abc': 0})
+        await client.get_database(
+            "mydb2", write_concern=wc_majority).bar.insert_one({'xyz': 0})
+
+        # Step 1: Define the callback that specifies the sequence of operations to perform inside the transactions.
+        async def callback(my_session):
+            collection_one = my_session.client.mydb1.foo
+            collection_two = my_session.client.mydb2.bar
+
+            # Important:: You must pass the session to the operations.
+            await collection_one.insert_one({'abc': 1}, session=my_session)
+            await collection_two.insert_one({'xyz': 999}, session=my_session)
+
+        # Step 2: Start a client session.
+        async with await client.start_session() as session:
+            # Step 3: Use with_transaction to start a transaction, execute the callback, and commit (or abort on error).
+            await session.with_transaction(
+                callback, read_concern=ReadConcern('local'),
+                write_concern=wc_majority,
+                read_preference=ReadPreference.PRIMARY)
+
+        # End Transactions withTxn API Example 1
+
+    @env.require_version_min(3, 6)
+    @env.require_replica_set
+    @asyncio_test
+    async def test_causal_consistency(self):
+        # Causal consistency examples
+        client = self.cx
+        self.addCleanup(env.sync_cx.drop_database, 'test')
+        await client.test.drop_collection('items')
+        await client.test.items.insert_one({
+            'sku': "111", 'name': 'Peanuts',
+            'start': datetime.datetime.today()})
+
+        # Start Causal Consistency Example 1
+        async with await client.start_session(causal_consistency=True) as s1:
+            current_date = datetime.datetime.today()
+            items = client.get_database(
+                'test', read_concern=ReadConcern('majority'),
+                write_concern=WriteConcern('majority', wtimeout=1000)).items
+            await items.update_one(
+                {'sku': "111", 'end': None},
+                {'$set': {'end': current_date}}, session=s1)
+            await items.insert_one(
+                {'sku': "nuts-111", 'name': "Pecans",
+                 'start': current_date}, session=s1)
+        # End Causal Consistency Example 1
+
+        # Start Causal Consistency Example 2
+        async with await client.start_session(causal_consistency=True) as s2:
+            s2.advance_cluster_time(s1.cluster_time)
+            s2.advance_operation_time(s1.operation_time)
+
+            items = client.get_database(
+                'test', read_preference=ReadPreference.SECONDARY,
+                read_concern=ReadConcern('majority'),
+                write_concern=WriteConcern('majority', wtimeout=1000)).items
+            async for item in items.find({'end': None}, session=s2):
+                print(item)
+        # End Causal Consistency Example 2
