@@ -20,10 +20,12 @@ import concurrent.futures
 from unittest import SkipTest
 
 from mockupdb import MockupDB
+from bson import SON
 from tornado import gen, testing
 
 import motor
 from test.test_environment import env, CA_PEM, CLIENT_PEM
+from test.version import Version
 
 
 @gen.coroutine
@@ -84,6 +86,12 @@ class MotorTest(testing.AsyncTestCase):
 
     make_test_data.__test__ = False
 
+    @gen.coroutine
+    def set_fail_point(self, client, command_args):
+        cmd = SON([('configureFailPoint', 'failCommand')])
+        cmd.update(command_args)
+        yield client.admin.command(cmd)
+
     def get_client_kwargs(self, **kwargs):
         if env.mongod_started_with_ssl:
             kwargs.setdefault('ssl_certfile', CLIENT_PEM)
@@ -151,3 +159,15 @@ class MotorMockServerTest(MotorTest):
 
     def run_thread(self, fn, *args, **kwargs):
         return self.executor.submit(fn, *args, **kwargs)
+
+
+class AsyncVersion(Version):
+    """Version class that can be instantiated with an async client from
+    within a coroutine."""
+    @classmethod
+    @gen.coroutine
+    def from_client(cls, client):
+        info = yield client.server_info()
+        if 'versionArray' in info:
+            raise gen.Return(cls.from_version_array(info['versionArray']))
+        raise gen.Return(cls.from_string(info['version']))
