@@ -23,10 +23,10 @@ import unittest
 import bson
 from bson import CodecOptions
 from bson.binary import JAVA_LEGACY
-from bson.objectid import ObjectId
 from pymongo import ReadPreference, WriteConcern
+from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import Secondary
-from pymongo.errors import DuplicateKeyError, OperationFailure
+from pymongo.errors import BulkWriteError, DuplicateKeyError, OperationFailure
 from tornado import gen
 from tornado.concurrent import Future
 from tornado.testing import gen_test
@@ -90,13 +90,11 @@ class MotorCollectionTest(MotorTest):
     @gen_test
     def test_update(self):
         yield self.collection.insert_one({'_id': 1})
-        result = yield self.collection.update(
+        result = yield self.collection.update_one(
             {'_id': 1}, {'$set': {'foo': 'bar'}})
 
-        self.assertEqual(1, result['ok'])
-        self.assertEqual(True, result['updatedExisting'])
-        self.assertEqual(1, result['n'])
-        self.assertEqual(None, result.get('err'))
+        self.assertIsNone(result.upserted_id)
+        self.assertEqual(1, result.modified_count)
 
     @ignore_deprecations
     @gen_test
@@ -108,7 +106,7 @@ class MotorCollectionTest(MotorTest):
         try:
             yield coll.insert_many([{'s': 1}, {'s': 2}])
             with self.assertRaises(DuplicateKeyError):
-                yield coll.update({'s': 2}, {'$set': {'s': 1}})
+                yield coll.update_one({'s': 2}, {'$set': {'s': 1}})
 
         finally:
             yield coll.drop()
@@ -126,8 +124,8 @@ class MotorCollectionTest(MotorTest):
         yield collection.insert_one({'_id': 2})
 
         # Violate a unique index in one of many updates, handle error.
-        with self.assertRaises(DuplicateKeyError):
-            yield collection.insert([
+        with self.assertRaises(BulkWriteError):
+            yield collection.insert_many([
                 {'_id': 1},
                 {'_id': 2},  # Already exists
                 {'_id': 3}])
@@ -137,46 +135,6 @@ class MotorCollectionTest(MotorTest):
             set([1, 2]),
             set((yield collection.distinct('_id'))))
 
-    @ignore_deprecations
-    @gen_test
-    def test_save_callback(self):
-        yield self.collection.save({}, callback=None)
-
-        # Should not raise
-        (result, error), _ = yield gen.Task(self.collection.save, {})
-        if error:
-            raise error
-
-    @ignore_deprecations
-    @gen_test
-    def test_save_with_id(self):
-        # save() returns the _id, in this case 5.
-        self.assertEqual(
-            5,
-            (yield self.collection.save({'_id': 5})))
-
-    @ignore_deprecations
-    @gen_test
-    def test_save_without_id(self):
-        collection = self.collection
-        result = yield collection.save({'fiddle': 'faddle'})
-
-        # save() returns the new _id
-        self.assertTrue(isinstance(result, ObjectId))
-
-    @ignore_deprecations
-    @gen_test
-    def test_save_bad(self):
-        coll = self.db.unique_collection
-        yield coll.ensure_index('s', unique=True)
-        yield coll.save({'s': 1})
-
-        try:
-            with self.assertRaises(DuplicateKeyError):
-                yield coll.save({'s': 1})
-        finally:
-            yield coll.drop()
-
     @gen_test
     def test_delete_one(self):
         # Remove a document twice, check that we get a success responses
@@ -196,78 +154,30 @@ class MotorCollectionTest(MotorTest):
         self.assertEqual(1, result.raw_result['ok'])
         self.assertEqual(None, result.raw_result.get('err'))
 
-    @ignore_deprecations
     @gen_test
     def test_unacknowledged_insert(self):
-        # Test that unsafe inserts with no callback still work
+        # Test that unacknowledged inserts complete eventually.
 
-        # Insert id 1 without a callback or w=1.
         coll = self.db.test_unacknowledged_insert
-        coll.with_options(write_concern=WriteConcern(0)).insert_one({'_id': 1})
+        yield coll.with_options(
+            write_concern=WriteConcern(0)).insert_one({'_id': 1})
 
         # The insert is eventually executed.
-        while not (yield coll.count()):
+        while not (yield coll.count_documents({})):
             yield gen.sleep(0.1)
 
-        # DuplicateKeyError not raised.
-        future = coll.insert({'_id': 1})
-        yield coll.insert({'_id': 1}, w=0)
-
-        with self.assertRaises(DuplicateKeyError):
-            yield future
-
-    @ignore_deprecations
-    @gen_test
-    def test_unacknowledged_save(self):
-        # Test that unsafe saves with no callback still work
-        collection_name = 'test_unacknowledged_save'
-        coll = self.db[collection_name]
-        coll.save({'_id': 201}, w=0)
-
-        while not (yield coll.find_one({'_id': 201})):
-            yield gen.sleep(0.1)
-
-        # DuplicateKeyError not raised
-        coll.save({'_id': 201})
-        yield coll.save({'_id': 201}, w=0)
-        coll.database.client.close()
-
-    @ignore_deprecations
     @gen_test
     def test_unacknowledged_update(self):
-        # Test that unsafe updates with no callback still work
+        # Test that unacknowledged updates complete eventually.
         coll = self.collection
 
         yield coll.insert_one({'_id': 1})
-        coll.update({'_id': 1}, {'$set': {'a': 1}}, w=0)
+        yield coll.with_options(write_concern=WriteConcern(0)).update_one(
+            {'_id': 1}, {'$set': {'a': 1}})
 
         while not (yield coll.find_one({'a': 1})):
             yield gen.sleep(0.1)
 
-        coll.database.client.close()
-
-    @gen_test(timeout=30)
-    def test_nested_callbacks(self):
-        results = [0]
-        future = Future()
-        yield self.collection.delete_many({})
-        yield self.collection.insert_one({'_id': 1})
-
-        def callback(result, error):
-            if error:
-                future.set_exception(error)
-            elif result:
-                results[0] += 1
-                if results[0] < 1000:
-                    self.collection.find({'_id': 1}).each(callback)
-                else:
-                    future.set_result(None)
-
-        self.collection.find({'_id': 1}).each(callback)
-
-        yield future
-        self.assertEqual(1000, results[0])
-
     @gen_test
     def test_map_reduce(self):
         # Count number of documents with even and odd _id
@@ -325,18 +235,6 @@ class MotorCollectionTest(MotorTest):
         index_info = yield test_collection.index_information()
         self.assertEqual([('foo', 1)], index_info[idx_name]['key'])
 
-        # Ensure the same index, test that callback is executed
-        result = yield test_collection.ensure_index([('foo', 1)])
-        self.assertEqual('foo_1', result)
-        result2 = yield test_collection.ensure_index([('foo', 1)])
-        self.assertEqual(None, result2)
-
-        # Ensure an index that doesn't exist, test it's created
-        yield test_collection.ensure_index([('bar', 1)])
-        index_info = yield test_collection.index_information()
-        self.assertTrue(any([
-            info['key'] == [('bar', 1)] for info in index_info.values()]))
-
         # Don't test drop_index or drop_indexes -- Synchro tests them
 
     @gen.coroutine
@@ -382,56 +280,6 @@ class MotorCollectionTest(MotorTest):
             self.assertTrue('_unpack_response' in formatted
                             or '_check_command_response' in formatted)
 
-    @gen_test(timeout=30)
-    def test_aggregation_cursor_to_list_callback(self):
-        db = self.db
-
-        # A small collection which returns only an initial batch,
-        # and a larger one that requires a getMore.
-        for collection_size in (10, 1000):
-            expected_sum = yield self._make_test_data(collection_size)
-            cursor = db.test.aggregate(self.pipeline)
-            future = Future()
-
-            def cb(result, error):
-                if error:
-                    future.set_exception(error)
-                else:
-                    future.set_result(result)
-
-            cursor.to_list(collection_size, callback=cb)
-            docs = yield future
-            self.assertAllDocs(expected_sum, docs)
-
-    @gen_test(timeout=30)
-    def test_parallel_scan(self):
-        yield skip_if_mongos(self.cx)
-
-        collection = self.collection.with_options(
-            write_concern=WriteConcern(test.env.w))
-
-        # Enough documents that each cursor requires multiple batches.
-        yield collection.delete_many({})
-        yield collection.insert_many(({'_id': i} for i in range(8000)))
-        if test.env.is_replica_set:
-            # Test that getMore messages are sent to the right server.
-            client = self.motor_rsc(read_preference=Secondary())
-            collection = client.motor_test.test_collection
-
-        docs = []
-
-        @gen.coroutine
-        def f(cursor):
-            self.assertTrue(isinstance(cursor,
-                                       motor.motor_tornado.MotorCommandCursor))
-
-            while (yield cursor.fetch_next):
-                docs.append(cursor.next_object())
-
-        cursors = yield collection.parallel_scan(3)
-        yield [f(cursor) for cursor in cursors]
-        self.assertEqual(len(docs), (yield collection.count()))
-
     def test_with_options(self):
         coll = self.db.test
         codec_options = CodecOptions(
@@ -452,6 +300,30 @@ class MotorCollectionTest(MotorTest):
         self.assertEqual(coll.codec_options, coll2.codec_options)
         self.assertEqual(coll.write_concern, coll2.write_concern)
 
+    def test_sub_collection(self):
+        # Verify that a collection with a dotted name inherits options from its
+        # parent collection.
+        write_concern = WriteConcern(w=2, j=True)
+        read_concern = ReadConcern("majority")
+        read_preference = Secondary([{"dc": "sf"}])
+        codec_options = CodecOptions(
+            tz_aware=True, uuid_representation=JAVA_LEGACY)
+
+        coll1 = self.db.get_collection(
+            'test',
+            write_concern=write_concern,
+            read_concern=read_concern,
+            read_preference=read_preference,
+            codec_options=codec_options)
+
+        coll2 = coll1.subcollection
+        coll3 = coll1['subcollection']
+
+        for c in [coll1, coll2, coll3]:
+            self.assertEqual(write_concern, c.write_concern)
+            self.assertEqual(read_concern, c.read_concern)
+            self.assertEqual(read_preference, c.read_preference)
+            self.assertEqual(codec_options, c.codec_options)
 
 if __name__ == '__main__':
     unittest.main()
