@@ -16,10 +16,11 @@ from __future__ import unicode_literals
 
 """Validate list of PyMongo attributes wrapped by Motor."""
 
-from tornado.testing import gen_test
+from tornado.testing import gen_test, unittest
 from gridfs import GridFSBucket, GridIn
 
 from motor import MotorGridFSBucket, MotorGridIn
+from motor.core import PY35
 from test import env
 from test.tornado_tests import MotorTest
 
@@ -43,9 +44,13 @@ pymongo_client_only = set([
     'database_names',
     'is_locked',
     'set_cursor_manager',
-    'get_default_database',
     'kill_cursors']).union(pymongo_only)
 
+if PY35:
+    pymongo_client_session_only = set([])
+else:
+    pymongo_client_session_only = set(['with_transaction'])
+
 pymongo_database_only = set([
     'add_user',
     'collection_names',
@@ -92,12 +97,13 @@ class MotorCoreTest(MotorTest):
             attrs(env.sync_cx) - pymongo_client_only,
             attrs(self.cx) - motor_client_only)
 
+    @unittest.skip("Skip until MOTOR-444 is complete")
     @env.require_version_min(3, 6)
     @env.require_replica_set
     @gen_test
     def test_client_session_attrs(self):
         self.assertEqual(
-            attrs(env.sync_cx.start_session()),
+            attrs(env.sync_cx.start_session()) - pymongo_client_session_only,
             attrs((yield self.cx.start_session())) - motor_only)
 
     def test_database_attrs(self):
