@@ -23,6 +23,7 @@ from pymongo import monitoring
 
 import contextlib
 import functools
+import os
 import warnings
 
 
@@ -96,6 +97,10 @@ class TestListener(monitoring.CommandListener):
         else:
             return self.results['started'][0]
 
+    def started_command_names(self):
+        """Return list of command names started."""
+        return [event.command_name for event in self.results['started']]
+
 
 def session_ids(client):
     return [s.session_id for s in client.delegate._topology._session_pool]
@@ -109,3 +114,15 @@ def create_user(authdb, user, pwd=None, roles=None, **kwargs):
     cmd['roles'] = roles or ['root']
     cmd.update(**kwargs)
     return authdb.command(cmd)
+
+
+def get_async_test_timeout(default=5):
+    """Get the global timeout setting for async tests.
+
+    Returns a float, the timeout in seconds.
+    """
+    try:
+        timeout = float(os.environ.get('ASYNC_TEST_TIMEOUT'))
+        return max(timeout, default)
+    except (ValueError, TypeError):
+        return default
